@@ -1,5 +1,5 @@
 /*--------------------------------------------------------------------*/
-/*     Copyright (C) 2004-2015  Serge Iovleff, Université Lille 1, Inria
+/*     Copyright (C) 2004-2016  Serge Iovleff, Université Lille 1, Inria
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as
@@ -80,6 +80,11 @@
 #include "../projects/DManager/include/STK_DataHandlerBase.h"
 #include "../projects/DManager/include/STK_IDataHandler.h"
 
+/* Interface class for data wrappers */
+#include "../projects/DManager/include/STK_IDataBridge.h"
+#include "../projects/DManager/include/STK_DataBridge.h"
+
+
 /* main classes for managing Csv data. */
 #include "../projects/DManager/include/STK_ReadWriteCsv.h"
 #include "../projects/DManager/include/STK_DataHandler.h"
